/******************************************************************************
* C Include: File Names                                                       
*                                                                             
* File:                                                                       
*    filenams.h                                                               
*                                                                             
* Description:                                                                
*    Include file for functions used to determine attributes of filenames on  
*    on CD-ROM, and to create file names.                                     
*                                                                             
* History:                                                                    
*    Created    : 15-NOV-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 14-NOV-95 by J. Doggett,                                    
*       Added COPYRIGHT_FILE_NAME_ROOT macro.                                 
*       Added #ifdef to define the copyright file name root depending on if   
*          we're compiling for a jukebox NFS mounted on an OpenVMS machine.   
*       Added config.h include to support this new #ifdef.                    
*    Redelivered: 01-AUG-96 by J. Doggett,                                    
*       GetImage Version 1.3.                                                 
*                                                                             
* Copyright (c) 1993-1996, Association of Universities for Research in        
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#ifndef __FILENAMS__
#define __FILENAMS__

#include "config.h"
#include "generic.h"

/***********************
* File Names constants *
***********************/
#ifdef __VMS_NFS_JUKEBOX__
#   define COPYRIGHT_FILE_NAME_ROOT "copyrght$5n"
#else
#   define COPYRIGHT_FILE_NAME_ROOT "copyrght"
#endif

/***********************************
* File Names functions' prototypes *
***********************************/
STRING  Build_Full_File_Name     (/* STRING Directory    ,
                                     STRING Sub_Directory,
                                     STRING File          */);

STRING  Build_Full_VMS_File_Name (/* STRING Directory    ,
                                     STRING Sub_Directory,
                                     STRING File          */);

STRING  Build_Full_UNIX_File_Name(/* STRING Directory    ,
                                     STRING Sub_Directory,
                                     STRING File          */);

BOOL    CopyRight_File_Exists    (/* STRING Directory     */);

BOOL    FileNames_in_LowerCase   (/* STRING Directory     */);

/* *** File Names Include File ***                                         2 */


BOOL    FileNames_in_UpperCase   (/* STRING Directory     */);

BOOL    FileNames_have_Version   (/* STRING Directory     */);

BOOL    FileNames_Trailing_Period(/* STRING Directory     */);

BOOL    Set_FileNames_Case       (/* STRING Directory,
                                     STRING FileName      */);

STRING  Set_FileNames_Version    (/* STRING Directory,
                                     STRING FileName      */);

#endif
